"""
pkg/history - 사용 이력 관리

최근 사용, 즐겨찾기, 프로파일 그룹, 사용 통계 관리
"""

from .favorites import FavoriteItem, FavoritesManager
from .profile_groups import ProfileGroup, ProfileGroupsManager
from .recent import RecentHistory, RecentItem

__all__: list[str] = [
    "RecentHistory",
    "RecentItem",
    "FavoritesManager",
    "FavoriteItem",
    "ProfileGroupsManager",
    "ProfileGroup",
]
